#include "HostBattleWindowHostBattleDialogBase.h"

HostBattleWindowHostBattleDialogBase::HostBattleWindowHostBattleDialogBase( wxWindow* parent )
:
HostBattleDialogBase( parent )
{

}

void HostBattleWindowHostBattleDialogBase::OnEngineSelect( wxCommandEvent& event )
{
// TODO: Implement OnEngineSelect
}

void HostBattleWindowHostBattleDialogBase::OnUseRelay( wxCommandEvent& event )
{
// TODO: Implement OnUseRelay
}

void HostBattleWindowHostBattleDialogBase::OnPickRelayHost( wxCommandEvent& event )
{
// TODO: Implement OnPickRelayHost
}

void HostBattleWindowHostBattleDialogBase::OnNatChange( wxCommandEvent& event )
{
// TODO: Implement OnNatChange
}

void HostBattleWindowHostBattleDialogBase::OnCancel( wxCommandEvent& event )
{
// TODO: Implement OnCancel
}

void HostBattleWindowHostBattleDialogBase::OnOk( wxCommandEvent& event )
{
// TODO: Implement OnOk
}
