#ifndef __HostBattleWindowHostBattleDialogBase__
#define __HostBattleWindowHostBattleDialogBase__

/**
@file
Subclass of HostBattleDialogBase, which is generated by wxFormBuilder.
*/

#include "HostBattleDialogBase.h"

//// end generated include

/** Implementing HostBattleDialogBase */
class HostBattleWindowHostBattleDialogBase : public HostBattleDialogBase
{
	protected:
		// Handlers for HostBattleDialogBase events.
		void OnEngineSelect( wxCommandEvent& event );
		void OnUseRelay( wxCommandEvent& event );
		void OnPickRelayHost( wxCommandEvent& event );
		void OnNatChange( wxCommandEvent& event );
		void OnCancel( wxCommandEvent& event );
		void OnOk( wxCommandEvent& event );
	public:
		/** Constructor */
		HostBattleWindowHostBattleDialogBase( wxWindow* parent );
	//// end generated class members

};

#endif // __HostBattleWindowHostBattleDialogBase__
